/*
   Copyright 2014-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#include <assert.h>
#include <list>

namespace Zipr_SDK
{
	using namespace std;
	using namespace IRDB_SDK;

	class Dollop_t;
	class DollopManager_t;
	class DollopEntry_t;

	using DollopEntryList_t = list<DollopEntry_t*>;

	class Placeable_t 
	{
		protected:
			Placeable_t() {}
			Placeable_t(const Placeable_t& copy) = delete;
		public:
			virtual ~Placeable_t() {}
			virtual void           Place(RangeAddress_t place)        = 0;
			virtual RangeAddress_t getPlace()                  const  = 0;
			virtual bool           isPlaced()                  const  = 0;

		friend ostream &operator<<(ostream &, const Placeable_t &);
	};

	class DollopEntry_t : virtual public Placeable_t 
	{
		protected: 
			DollopEntry_t() { } 
			DollopEntry_t(const DollopEntry_t& copy) = delete;
		public:
			virtual ~DollopEntry_t() { } 

			virtual void           setTargetDollop(Dollop_t *target)   = 0 ; 
			virtual void           MemberOfDollop(Dollop_t *member_of) = 0 ; 

			virtual Instruction_t* getInstruction()    const = 0 ;
			virtual Dollop_t*      getTargetDollop()   const = 0 ; 
			virtual Dollop_t*      getMemberOfDollop() const = 0 ; 

			virtual bool operator==(const DollopEntry_t &);
			virtual bool operator!=(const DollopEntry_t &);

		friend ostream &operator<<(ostream &, const DollopEntry_t &);
	};


	class Dollop_t : virtual public Placeable_t, virtual public DollopEntryList_t
	{
		protected:
			Dollop_t() {}
			Dollop_t(const Dollop_t& copy) = delete;

		public:
			virtual ~Dollop_t() {}

			virtual size_t    getSize()                                  const = 0;
			virtual size_t    getDollopEntryCount()                      const = 0;
			virtual Dollop_t* getFallbackDollop()                        const = 0;
			virtual Dollop_t* getFallthroughDollop()                     const = 0;
			virtual bool      isFallthroughPatched()                     const = 0;
			virtual bool      wasTruncated()                             const = 0;
			virtual bool      wasCoalesced()                             const = 0;
			virtual DollopEntry_t* 
			                  getFallthroughDollopEntry(DollopEntry_t *) const = 0;

			virtual void setDollopManager(DollopManager_t *mgr)      = 0;
			virtual void setSize(size_t size)                        = 0;
			virtual void setCoalesced(bool coalesced)                = 0;
			virtual void setFallthroughPatched(bool patched)         = 0;
			virtual void setFallthroughDollop(Dollop_t *fallthrough) = 0;
			virtual void setFallbackDollop(Dollop_t *fallback)       = 0;
			virtual void setTruncated(bool trunc)                    = 0;

			virtual void reCalculateSize() = 0 ;
			virtual Dollop_t *split(Instruction_t *split_point) = 0 ;
			virtual void removeDollopEntries(DollopEntryList_t::iterator, DollopEntryList_t::iterator) = 0;

		friend ostream &operator<<(ostream &, const Dollop_t &);
	};
}
