/*
   Copyright 2014-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#include <set>
#include <map>
#include <list>


namespace Zipr_SDK 
{
	using namespace std;
	using namespace IRDB_SDK;


	using DollopList_t                 = set<Dollop_t*>;
	using InsnToDollopMap_t            = map<Instruction_t*,Dollop_t*>;


	class DollopManager_t 
	{
		protected:
			DollopManager_t() {};
			DollopManager_t(const DollopManager_t& copy) = delete;
		public:

			virtual ~DollopManager_t() {};

			/*
			 * Add a dollop and all its fallthrough dollops
			 * to the dollop manager.
			 */
			virtual void addDollops(Dollop_t *dollop_head)=0;

			/*
			 * Create new dollops from a particular start instruction.
			 */
			virtual Dollop_t *addNewDollops(Instruction_t *start)=0;

			/*
			 * Return the number of dollops under management.
			 */
			virtual size_t getSize()=0;

			/*
			 * Return the dollop containing an instruction.
			 */
			virtual Dollop_t *getContainingDollop( Instruction_t *insn)=0;

			/*
			 * Update the dollops under management based on the
			 * targetted dollops in a particular dollop.
			 */
			virtual bool updateTargets(Dollop_t *) = 0;
			/*
			 * Update all the dollops under management based on
			 * all the dollops of which the manager knows.
			 */
			virtual void updateAllTargets() = 0;

			/*
			 * Return beginning and ending iterators for all the
			 * dollops under management.
			 */
			virtual DollopList_t::iterator begin() = 0;
			virtual DollopList_t::iterator end() = 0;
			virtual DollopList_t::iterator dollops_begin() = 0;
			virtual DollopList_t::iterator dollops_end() = 0;

			/* Figure out how much space an instruction will take */
			virtual size_t determineDollopEntrySize(DollopEntry_t *entry)=0;

			/*
			 * print (for debugging) information about the dollop
			 * manager.
			 */
			friend ostream &operator<<(ostream &out, const DollopManager_t &dollop_man);
	};
}
