/*
   Copyright 2014-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

namespace Zipr_SDK
{
	using namespace std;

	using RangeSet_t       = set<Range_t, Range_tCompare> ;
	using RangeSetBounds_t = pair<RangeSet_t::const_iterator,RangeSet_t::const_iterator>;

	// a memory space _is_ a map of range addres to char, with additional functionality.
	class MemorySpace_t : virtual public map<RangeAddress_t,char>
	{
		protected:
			MemorySpace_t(){}
			MemorySpace_t(const MemorySpace_t& copy) = delete;

		public:
			virtual ~MemorySpace_t(){}

			// range operatations
			virtual void             splitFreeRange(RangeAddress_t addr)                              = 0;
			virtual void             splitFreeRange(Range_t range)                                    = 0;
			virtual void             mergeFreeRange(RangeAddress_t addr)                              = 0;
			virtual void             mergeFreeRange(Range_t range)                                    = 0;
			virtual RangeSet_t::iterator 
			                         findFreeRange(RangeAddress_t addr)                               = 0;
			virtual Range_t          getFreeRange(int size)                                           = 0;
			virtual Range_t          getInfiniteFreeRange()                                           = 0;
			virtual list<Range_t>    getFreeRanges(size_t size = 0)                                   = 0;
			virtual void             addFreeRange(Range_t newRange)                                   = 0;
			virtual void             removeFreeRange(Range_t newRange)                                = 0;
			virtual Range_t          getLargeRange(void)                                              = 0;

			// queries about free areas.
			virtual bool             areBytesFree(RangeAddress_t addr, int num_bytes)                 = 0;
			virtual bool             isByteFree(RangeAddress_t addr)                                  = 0;
			virtual bool             isValidRange(RangeSet_t::iterator it)                            = 0;
			virtual int              getRangeCount()                                                  = 0;
			virtual void             printMemorySpace(ostream &out)                                   = 0;

			virtual void             plopBytes(RangeAddress_t addr, const char the_byte[], int num)   = 0;
			virtual void             plopByte(RangeAddress_t addr, char the_byte)                     = 0;
			virtual void             plopJump(RangeAddress_t addr)                                    = 0;
			virtual RangeAddress_t   getMinPlopped() const                                            = 0;
			virtual RangeAddress_t   getMaxPlopped() const                                            = 0;

	};

}

