/*
   Copyright 2014-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

namespace Zipr_SDK
{
	using namespace std;

	using RangeAddress_t = uintptr_t ;

	// Note: Unlike other Zipr SDK classes, this is copyable, assignable, etc.
	//       i.e., this is not a pure virtual class.
	class Range_t
	{
		public:
			Range_t(RangeAddress_t p_s, RangeAddress_t p_e);
			Range_t();	 // zero init

			virtual ~Range_t();

			virtual RangeAddress_t getStart() const;
			virtual RangeAddress_t getEnd()   const;
			virtual size_t getSize()          const;
			virtual bool   is2ByteRange()     const;
			virtual bool   is5ByteRange()     const;
			virtual bool   isInfiniteRange()  const;

			virtual void setStart(RangeAddress_t s);
			virtual void setEnd(RangeAddress_t e);

		protected:
			RangeAddress_t m_start, m_end;
	};

	// useful for putting Range_t's in containers (sets, etc).
	struct Range_tCompare
	{
		// Caution!  Only works for NON-OVERLAPPING ranges.
		bool operator() (const Range_t first, const Range_t second) const;
	};

}
