/*
   Copyright 2014-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#include <set>
#include <utility>


namespace Zipr_SDK
{
	using namespace std;

	using PlacementQueue_t = set< pair<Dollop_t*,RangeAddress_t> > ;

	class ZiprOptionsManager_t;

	class Zipr_t
	{
		protected:
			Zipr_t() { } 
			Zipr_t(const Zipr_t& copy) = delete;

		public:
			virtual ~Zipr_t() { } 
			/*
			 * These are public functions that the SDK user way want to use.
			 */
			virtual size_t determineDollopEntrySize(
					DollopEntry_t*, 
					bool account_for_fallthrough)         = 0;

			virtual void applyPatch(
					RangeAddress_t from_addr, 
					RangeAddress_t to_addr)               = 0;

			virtual MemorySpace_t*            getMemorySpace()    = 0;
			virtual IRDB_SDK::FileIR_t*       getFileIR()         = 0;
			virtual InstructionLocationMap_t* getLocationMap()    = 0; 
			virtual DollopManager_t*          getDollopManager()  = 0;
			virtual PlacementQueue_t*         getPlacementQueue() = 0;
			virtual ZiprOptionsManager_t*     getOptionsManager() = 0;

	};

};

